<?php

namespace Drupal\smart_content_segments\Form;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\smart_content\Condition\ConditionManager;
use Drupal\smart_content\Entity\SmartVariationSet;
use Drupal\smart_content\Form\SegmentSetConfigEntityForm;
use Drupal\smart_content\Form\SmartVariationSetForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\smart_content\Condition\ConditionInterface;

/**
 * Class SmartSegmentForm.
 */
class SmartSegmentForm extends EntityForm {

  /**
   * Condition manager.
   *
   * @var \Drupal\smart_content\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * Conditions.
   *
   * @var array
   */
  protected $conditions;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConditionManager $condition_manager) {
    $this->conditionManager = $condition_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
    // Load the service required to construct this class.
      $container->get('plugin.manager.smart_content.condition')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form['#process'][] = [$this, 'buildWidget'];
    return $form;
  }



  /**
   * Render API callback: builds the formatter settings elements.
   */
  public function buildWidget(array &$element, FormStateInterface $form_state, array &$complete_form) {
    $entity = $this->getEntity();
    if (count($entity->getConditions()) === 0) {
      $entity->appendCondition($this->conditionManager->createInstance('group'));
    }

    $element['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Segment Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#required' => TRUE,
    ];

    $element['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\smart_content_segments\Entity\SmartSegment::load',
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $element['condition_settings'] = [
      '#type' => 'fieldset',
      '#attributes' => [
        'class' => [
          'decision-settings-wrapper',
        ],
      ],
      '#title' => $this->t('Configure the conditions for this segment'),
    ];
    $condition_storage = $form_state->get('conditions');
    foreach ($entity->getConditions() as $ii => $condition) {
      if (isset($condition_storage[$ii])) {
        $condition = $condition_storage[$ii];
        $entity->setCondition($ii, $condition);
      }
      SegmentSetConfigEntityForm::pluginForm($condition, $element, $form_state, ['condition_settings', $ii]);
      $condition_storage[$ii] = $condition;
      $form_state->set('conditions', $condition_storage);
    }


    $element['#attached']['library'][] = 'smart_content/form';


    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $entity = $this->getEntity();
    foreach ($entity->getConditions() as $condition_id => $condition) {
      SegmentSetConfigEntityForm::pluginFormValidate($condition, $form, $form_state, ['condition_settings', $condition_id]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    $entity = $this->getEntity();
    foreach ($entity->getConditions() as $condition_id => $condition) {
      SegmentSetConfigEntityForm::pluginFormSubmit($condition, $form, $form_state,  ['condition_settings', $condition_id]);
    }
    $status = $this->entity->save();
    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label Smart segment.', [
          '%label' => $entity->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the %label Smart segment.', [
          '%label' => $entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($entity->toUrl('collection'));
  }

}
