<?php

namespace Drupal\smart_content_segments\Plugin\smart_content\Condition;

use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\smart_content\Condition\ConditionBase;
use Drupal\smart_content\Condition\ConditionConfigurableBase;
use Drupal\smart_content\Condition\ConditionInterface;
use Drupal\smart_content\Condition\Group\ConditionGroupManager;
use Drupal\smart_content_segments\Entity\SmartSegment;

/**
 * Provides a Smart Segment condition plugin.
 *
 * @SmartCondition(
 *   id = "smart_segment",
 *   label = @Translation("Smart Segment"),
 *   group = "segment",
 *   deriver = "Drupal\smart_content_segments\Plugin\Derivative\SegmentDeriver"
 * )
 */
class Segment extends ConditionConfigurableBase implements CacheableDependencyInterface {

  /**
   * The Smart segment entity.
   *
   * @var \Drupal\smart_content_segments\Entity\SmartSegment
   */
  protected $entity;

  /**
   * {@inheritdoc}
   */
  public function getEntity() {
    return $this->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getTypeId() {
    return 'plugin:' . $this->getBaseId(); // TODO: Change the autogenerated
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConditionGroupManager $conditionGroupManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $conditionGroupManager);
    $entity_id = explode(':', $this->getPluginId())[1];
    $this->entity = SmartSegment::load($entity_id);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $configuration = $this->getConfiguration();
    $form = ConditionBase::attachNegateElement($form, $configuration);
    $condition_definition = $this->getPluginDefinition();

    $form['#attributes']['class'][] = 'condition';
    $form['label'] = [
      '#type' => 'container',
      '#markup' => $condition_definition['label'] . '(' . $condition_definition['group'] . ')',
      '#attributes' => [
        'class' => ['condition-label'],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setNegated($form_state->getValue('negate'));
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    $configuration = parent::getConfiguration();
    $configuration['segment_id'] =  $this->getEntity()->id();
    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function getAttachedSettings() {
    $settings = parent::getAttachedSettings();
    $settings['conditions'] = array_map(function (ConditionInterface $condition) {
      return $condition->getAttachedSettings();
    }, $this->getEntity()->getConditions()->getIterator()->getArrayCopy());
    $settings['settings']['negate'] = $this->isNegated();
    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries() {
    $libraries = array_map(function (ConditionInterface $condition) {
      return $condition->getLibraries();
    }, $this->getEntity()->getConditions()->getIterator()->getArrayCopy());

    return array_unique(array_merge(parent::getLibraries(), array_reduce($libraries, 'array_merge', []), [
      'smart_content/condition.common',
      'smart_content/condition_type.standard',
      'smart_content_segments/condition.smart_segment',
    ]));
  }

  /**
   * {@inheritdoc}
   */
  public function getHtmlSummary() {
    $segment_id = $this->getConfiguration()['segment_id'];
    $output = 'Segment';
    if ($entity = SmartSegment::load($segment_id)) {
      $output.= ': ' . $entity->label();
    }
    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return $this->getEntity()->getCacheContexts();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return $this->getEntity()->getCacheTags();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return $this->getEntity()->getCacheMaxAge();
  }

}
